var laraHelpers    = {};
(function(lH){
	"use strict";
	
	lH.debug   = function(){};
	lH.setDebugMode = function(enable =false){
		if(enable){
			lH.debug = console.log.bind(JSON.parse(JSON.stringify(console)));
		}else{
			lH.debug   = function(){};
		}
	}

	lH.secondsToHHMMSS = function (seconds) {
	  const days = Math.floor(seconds / 86400);
	  const hours = Math.floor((seconds % 86400) / 3600);
	  const minutes = Math.floor((seconds % 3600) / 60);
	  seconds = seconds % 60;

	  let formattedDuration = '';

	  if (days > 0) {
		formattedDuration += String(days * 24 + hours).padStart(2, '0') + ':';
	  } else if (hours > 0) {
		formattedDuration += String(hours).padStart(2, '0') + ':';
	  }

	  formattedDuration += String(minutes).padStart(2, '0') + ':' + String(seconds).padStart(2, '0');

	  return formattedDuration;
	}
	
	lH.doCall = function(call,type = ""){
		$("#lraias_ticket_overlay").show();
		$("[data-lraias-ticket-request]").attr("disabled", "disabled");
		$("#lraias_errors").hide();
		var cType = "application/x-www-form-urlencoded; charset=UTF-8";
		if(type == "json"){
			call  = JSON.stringify(call);
			cType = "application/json; charset=utf-8";
		}
			
		return $.ajax({
			method: "POST",
			url: "addonmodules.php?module=lraias",
			data: call,
			contentType: cType,
			dataType: 'json'
		})
		.done(function (data, textStatus, jqXHR) {
			lH.debug(data);
			if (data.status !== "success"){
				if (typeof data.error !== 'undefined' && data.error) {
					$("#lraias_errors_container .lraias_errors_error").html(data.error);
				}
				
				if (typeof data.description !== 'undefined' && data.description) {
					$("#lraias_errors_container .lraias_errors_description").html(data.description);
				}
				
				if (typeof data.code !== 'undefined' && data.code) {
					$("#lraias_errors_container .lraias_errors_code").html(data.code);
					$("#lraias_errors_container .lraias_errors_code_cont").show();
				}
				
				if (typeof data.remaining_lock_time !== 'undefined' && data.remaining_lock_time) {
					$("#lraias_errors_container .lraias_errors_remaining_lock_time").html(lH.secondsToHHMMSS(data.remaining_lock_time));
					$("#lraias_errors_container .lraias_errors_remaining_lock_time_cont").show();
				}else{
					$("#lraias_errors_container .lraias_errors_remaining_lock_time_cont").hide();
				}
				
				if (typeof data.debug !== 'undefined' && data.debug) {
					$("#lraias_errors_container .lraias_errors_debug").html(JSON.stringify(data.debug, null, 2));
					$("#lraias_errors_container .lraias_errors_debug").show();
				}
				
				if (typeof data.type !== 'undefined' && data.type){
					if(data.type === "info"){
						$("#lraias_errors_container").css({"background-color":"#00c0ef"});
					}else{$("#lraias_errors_container").css({"background-color":""});}
				}
				
				$("#lraias_errors").show();
			}			
			lraiasadmininterface.setStatistics(data);
		})
		.fail(function (jqXHR, textStatus, errorThrown) {
			lH.debug(jqXHR);
			lH.debug(textStatus);
			lH.debug(errorThrown);
		})		
		.always(function (dataOrjqXHR, textStatus, jqXHRorErrorThrown) {
			lH.debug(textStatus);
			$("#lraias_ticket_overlay").hide();
			$("[data-lraias-ticket-request]").removeAttr("disabled");
		});
	}	

})(laraHelpers);


var lraiasadmininterface = {};
(function(LRETAI){
	"use strict";
	
	let lraiasids = [];
	
	function getURLParameter(url, name) {
		return (RegExp(name + '=' + '(.+?)(&|$)').exec(url)||[,null])[1];
	}
	
	function format_timestamp(ts){
		if (typeof ts !== 'undefined' && ts) {
			return moment.unix(ts).format('MMMM D, YYYY , h:mm:ss a');
		}else{return '-';}
	}
	
	function set_reply(reply = ""){
		$("textarea#replymessage").val(reply).trigger("change");
	}
	
	LRETAI.setStatistics = function (data){
		if(typeof data.statistics !== 'undefined' && data.statistics){
			if(typeof data.statistics.remaining_credits !== 'undefined' && Number.isInteger(data.statistics.remaining_credits)){
					$("#remaining_credits").text(data.statistics.remaining_credits);
			}
		}
	}	

	LRETAI.construct = function (){
		$(document).ready(function() {
			try {
				if (typeof (LRAIAS_OUTPUT_TYPE) !== "undefined"){
					if (LRAIAS_OUTPUT_TYPE === "viewticket"){
						let ticket_id = $("#ticketreplies input[name=id]").val();
						let buttons = '<div class="lraias_ticket_btns_container">\
											<div class="btn-group">\
												<div class="btn-default btn-sm btn lraias_tooltip" data-lraias-information="Generate an answer" data-lraias-ticket-request="generate" data-lraias-ticket-id="'+ticket_id+'"><i class="fas fa-reply"></i></div>\
											</div>\
											<div class="btn-group">\
												<div class="btn-default btn-sm btn lraias_tooltip" data-lraias-information="Rewrite the answer" data-lraias-ticket-request="rewrite" data-lraias-ticket-id="'+ticket_id+'"><i class="fas fa-sync-alt"></i></div>\
											</div>\
											<div class="btn-group">\
												<div class="btn-default btn-sm btn lraias_tooltip" data-lraias-information="Task in progress" data-lraias-ticket-request="inprogress" data-lraias-ticket-id="'+ticket_id+'"><i class="fas fa-ellipsis-h"></i></div>\
												<div class="btn-default btn-sm btn lraias_tooltip" data-lraias-information="Task completed" data-lraias-ticket-request="completed" data-lraias-ticket-id="'+ticket_id+'"><i class="fas fa-check-square"></i></div>\
												<div class="btn-default btn-sm btn lraias_tooltip" data-lraias-information="Task declined" data-lraias-ticket-request="declined" data-lraias-ticket-id="'+ticket_id+'"><i class="fas fa-times-square"></i></div>\
											</div>\
											<div class="btn-group">\
												<div class="btn-default btn-sm btn lraias_tooltip" data-lraias-information="Delete the answer" data-lraias-ticket-request="delete" data-lraias-ticket-id="'+ticket_id+'"><i class="fas fa-trash"></i></div>\
											</div>\
											<div class="btn-group">\
												<a href="addonmodules.php?module=lraias&action=show_documentation" target="_blank"><div class="btn-default btn-sm btn lraias_tooltip" data-lraias-information="Documentation" ><i class="fas fa-info-circle"></i></div></a>\
											</div>\
											<div class="btn-default btn-sm btn lraias_tooltip" data-lraias-information="Remaining credits" id="remaining_credits" disabled>'+LRAIAS_MODULE_REMAINING_CREDITS+'</div>\
									    </div>';
						$("#replyingAdminMsg").after('<div id="lraias_errors" style="display:none;">\
														<div id="lraias_errors_container">\
															<div class="lraias_errors_icon"><i class="icon fas fa-exclamation-triangle fa-4x"></i></div>\
															<div class="lraias_errors_body">\
																<div class="lraias_errors_error"></div>\
																<div class="lraias_errors_description"></div>\
																<div class="lraias_errors_code_cont" style="display:none;">Error Code : <span class="lraias_errors_code"></span></div>\
																<div class="lraias_errors_remaining_lock_time_cont" style="display:none;">Please retry after : <span class="lraias_errors_remaining_lock_time"></span></div>\
																<div class="lraias_errors_debug" style="display:none;"></div>\
															</div>\
														</div>\
													</div>');
						$("#frmAddTicketReply .btn-toolbar").append(buttons);
						let AILoadingIcon = '<span class="lraias_ailoading fa-stack fa-2x"><i class="fad fa-network-wired fa-stack-1x"></i><i class="fal fa-spinner-third fa-stack-2x fa-spin"></i></span>';
						$("#frmAddTicketReply .md-editor").css("position","relative").prepend('<div id="lraias_ticket_overlay">' + AILoadingIcon + '</div>');			
						$("[data-lraias-ticket-request]").on( "click", function(e) {
							let t_request   = $(this).attr("data-lraias-ticket-request");
							let t_id        = $(this).attr("data-lraias-ticket-id");
							let t_rewrite   = "";
							if (t_request === "delete"){
								set_reply();
							}else{
								if (t_request === "rewrite"){
									t_rewrite = $("#frmAddTicketReply #replymessage").val(); //show error if empty
								}
								laraHelpers.doCall({action: "processTicket", request: t_request, id: t_id, rewrite: t_rewrite}).done(function (data, textStatus, jqXHR) {
									if (data.status === "success"){
										set_reply(data.output.query_results);
									}
								});							
							}

						});							
					}
				}
			}catch(e){console.log(e);}
		});
	}
	LRETAI.construct();
})(lraiasadmininterface);