var laraHelpers    = {};
(function(lH){
	"use strict";
	
	let isSessionStorageReady = false;
	
	if (typeof (sessionStorage) !== "undefined"){
		isSessionStorageReady = true;
	}	
	
	lH.debug   = function(){};
	
	lH.isObject = function (obj){
		if (typeof obj === 'object' && obj !== null){
			return true;
		}
	}
	
	lH.setDebugMode = function(enable =false){
		if(enable){
			lH.debug = console.log.bind(JSON.parse(JSON.stringify(console)));
		}else{
			lH.debug   = function(){};
		}
		
	}
	
	lH.getObjFromCache = function(name){
		if (isSessionStorageReady === true){
			try {
				return sessionStorage.getItem(name);
			}catch(e){laraHelpers.debug(e);return null;}
		}else{return null;}
	}
	
	lH.saveObjToCache = function(name, obj){
		if (isSessionStorageReady === true){
			try {
				return sessionStorage.setItem(name, obj);
			}catch(e){laraHelpers.debug(e);return null;}
		}else{return null;}	
	}
	
	lH.removeObjFromCache = function(name){
		if (isSessionStorageReady === true){
			try {
				return sessionStorage.removeItem(name);
			}catch(e){laraHelpers.debug(e);return null;}
		}else{return null;}	
	}

	lH.secondsToHHMMSS = function (seconds) {
	  const days = Math.floor(seconds / 86400);
	  const hours = Math.floor((seconds % 86400) / 3600);
	  const minutes = Math.floor((seconds % 3600) / 60);
	  seconds = seconds % 60;

	  let formattedDuration = '';

	  if (days > 0) {
		formattedDuration += String(days * 24 + hours).padStart(2, '0') + ':';
	  } else if (hours > 0) {
		formattedDuration += String(hours).padStart(2, '0') + ':';
	  }

	  formattedDuration += String(minutes).padStart(2, '0') + ':' + String(seconds).padStart(2, '0');

	  return formattedDuration;
	}

	lH.formToObject = function (form_id) {
		const form = document.getElementById(form_id)
		const formData = new FormData(form);
		const obj = {};

		for (const [key, value] of formData.entries()) {
			const keys = key.split(/\[|\]/).filter(k => k !== '');
			let current = obj;

			for (let i = 0; i < keys.length; i++) {
				const isLastKey = i === keys.length - 1;
				const currentKey = keys[i];

				if (isLastKey) {
					if (current[currentKey]) {
						if (!Array.isArray(current[currentKey])) {
							current[currentKey] = [current[currentKey]];
						}
						current[currentKey].push(value);
					} else {
						current[currentKey] = value;
					}
				} else {
					if (!current[currentKey]) {
						current[currentKey] = {};
					}
					current = current[currentKey];
				}
			}
		}

		return obj;
	}	
	
	lH.formatDate = function (timestamp) {
	  const monthNames = [
		"January", "February", "March", "April", "May", "June",
		"July", "August", "September", "October", "November", "December"
	  ];

	  const date = new Date(timestamp * 1000); // Convert the timestamp to milliseconds
	  const day = date.getDate();
	  const month = date.getMonth();
	  const year = date.getFullYear();
	  const monthName = monthNames[month];

	  return `${day} ${monthName} ${year}`;
	}

	lH.doCall = function(dest,call,type = ""){
		var cType = "application/x-www-form-urlencoded; charset=UTF-8";
		if(type == "json"){
			call  = JSON.stringify(call);
			cType = "application/json; charset=utf-8";
		}
			
		return $.ajax({
			method: "POST",
			url: dest,
			data: call,
			contentType: cType,
			dataType: 'json'
		})
		.done(function (data, textStatus, jqXHR) {
			lH.debug(data);
		})
		.fail(function (jqXHR, textStatus, errorThrown) {
			lH.debug(jqXHR);
			lH.debug(textStatus);
			lH.debug(errorThrown);
		})		
		.always(function (dataOrjqXHR, textStatus, jqXHRorErrorThrown) {

		});
	}	

})(laraHelpers);


var laraAjax            = {};
var laraMain            = {};
var laraMainFunctions   = {};


(function(lA){
	"use strict";
	
	lA.doCall = function(dest,call,type,loader = true){
		$("#lraias_loading").html('<i class="fa fa-spinner fa-pulse"></i>');
		$("#lraias_errors").hide();

		if(loader === true){$("#lraias_loader").show();}
		
		return laraHelpers.doCall(dest,call,type)
		.done(function (data, textStatus, jqXHR) {
			if (data.status !== "success"){
				if (typeof data.error !== 'undefined' && data.error) {
					$("#lraias_errors_container .lraias_errors_error").html(data.error);
				}
				
				if (typeof data.description !== 'undefined' && data.description) {
					$("#lraias_errors_container .lraias_errors_description").html(data.description);
				}
				
				if (typeof data.code !== 'undefined' && data.code) {
					$("#lraias_errors_container .lraias_errors_code").html(data.code);
					$("#lraias_errors_container .lraias_errors_code_cont").show();
					if (data.code.startsWith("LIC-")){
						$("#lraias_check_license").show();
					}else{
						$("#lraias_check_license").hide();
					}					
				}
				
				if (typeof data.remaining_lock_time !== 'undefined' && data.remaining_lock_time) {
					$("#lraias_errors_container .lraias_errors_remaining_lock_time").html(laraHelpers.secondsToHHMMSS(data.remaining_lock_time));
					$("#lraias_errors_container .lraias_errors_remaining_lock_time_cont").show();
				}else{
					$("#lraias_errors_container .lraias_errors_remaining_lock_time_cont").hide();
				}
				
				if (typeof data.debug !== 'undefined' && data.debug) {
					$("#lraias_errors_container .lraias_errors_debug").html(JSON.stringify(data.debug, null, 2));
					$("#lraias_errors_container .lraias_errors_debug").show();
				}

				if (typeof data.type !== 'undefined' && data.type){
					if(data.type === "info"){
						$("#lraias_errors_container").css({"background-color":"#00c0ef"});
					}else{$("#lraias_errors_container").css({"background-color":""});}
				}
				
				$("#lraias_errors").show();
			}
			laraMainFunctions.setStatistics(data);
		})
		.always(function (dataOrjqXHR, textStatus, jqXHRorErrorThrown) {
			$("#lraias_loading").html('');
			$("#lraias_loader").hide();
			
		});
	}

})(laraAjax);

(function(lMF){
	"use strict";
	
	lMF.setStatistics = function (data){
		$(".lraias-badge-number").html("N/A");
		$(".lraias-badge-alert-icon").hide();
		if (typeof data.statistics !== 'undefined' && data.statistics) {
			if(data.statistics.license_status){
				$(".lraias-badge-license_status .lraias-badge-number").html(data.statistics.license_status);
				if(data.statistics.license_status !== "active"){
					$(".lraias-badge-license_status .lraias-badge-alert-icon").show();
					$(".lraias-badge-license_status").addClass("inactive");
				}else{
					$(".lraias-badge-license_status").removeClass("inactive");
					
					if(typeof data.statistics.remaining_credits !== 'undefined' && typeof data.statistics.plan_limit !== 'undefined'){	
						if(Number.isInteger(data.statistics.remaining_credits) && Number.isInteger(data.statistics.plan_limit)){
							let remaining_percentage = 0;
							if(data.statistics.plan_limit > 0){
								remaining_percentage = Math.round((data.statistics.remaining_credits / data.statistics.plan_limit) * 100);
							}
							if(remaining_percentage < 20){
								$(".lraias-badge-remaining_credits .lraias-badge-alert-icon").show();
							}
							$(".lraias-badge-remaining_credits .lraias-badge-number").html(data.statistics.remaining_credits);
						}
					}
					

					if(typeof data.statistics.system_documents !== 'undefined' && typeof data.statistics.synced_documents !== 'undefined'){	
						if(typeof data.statistics.system_documents.total !== 'undefined' && typeof data.statistics.synced_documents.total !== 'undefined'){
							if(Number.isInteger(data.statistics.system_documents.total) && Number.isInteger(data.statistics.synced_documents.total)){
								
								if(typeof data.batch_progress === 'undefined'){
									let completed = 0;
									if(data.statistics.system_documents.total > 0){
										completed = Math.round((data.statistics.synced_documents.total / data.statistics.system_documents.total) * 100);
										lMF.update_kb_progress_bar(completed, data.statistics.synced_documents.total, data.statistics.system_documents.total, "none");									
									}									
								}
									
								if (data.statistics.system_documents.total !== data.statistics.synced_documents.total){
									$(".lraias-badge-synced_documents .lraias-badge-alert-icon").show();
								}
								
								$(".lraias-badge-synced_documents .lraias-badge-number").html(data.statistics.synced_documents.total + " / " + data.statistics.system_documents.total);
							}
						}
					}
						

					if(typeof data.statistics.credits_reset_date !== 'undefined' && data.statistics.credits_reset_date){
						$(".lraias-badge-credits_reset_date .lraias-badge-number").html(laraHelpers.formatDate(data.statistics.credits_reset_date));
					}
			
				}
			}
			
		}
	}
	
	lMF.update_kb_progress_bar = function (percentage, completed, total, transition = ""){
		let color, pt, txt, wdth;
		color = "#357ca5";
		pt    = percentage
		txt   =  wdth = pt + '%';
		if ( completed > total ){
			pt   = 100;
			wdth = pt + '%';
			txt  = "OUT OF SYNC";
			color = "#f56954"; 
		}
		$('#lraias_sync_kb_progress_bar').css('transition', transition)
										 .css('background-color', color)
										 .attr('aria-valuenow', pt)
										 .css('width', wdth)
										 .text(txt);
		$('#lraias_sync_kb_processing').text(' [ ' + completed + ' / ' + total +' ] ');	
	}
		
	let batch_status = "none"
	lMF.syncKB = function (){
		$('#lraias_sync_kb').prop('disabled', true);
		$('#lraias_sync_kb i').addClass("fa-pulse");
		$('#lraias_sync_kb_processing_cont').show();
		if(batch_status === "none"){lMF.update_kb_progress_bar(0, 0, 0, "none");}
		batch_status = "running";
		laraAjax.doCall(LRAIAS_MODULE_ADMIN_LINK, {action: "syncKB"}).done(function (data, textStatus, jqXHR) {
			if (data.status === "success"){
				lMF.update_kb_progress_bar(data.batch_progress.percentage, data.batch_progress.completed, data.batch_progress.total);
				if (data.batch_progress.status !== "done" ){
					if (data.batch_progress.status === "sync" ){$('#lraias_sync_kb_sync_msg').show();}
					setTimeout(function(){
						lMF.syncKB();
					}, 2000);			
					
				}else{
					$('#lraias_sync_kb').prop('disabled', false);
					$('#lraias_sync_kb i').removeClass("fa-pulse");
					$('#lraias_sync_kb_sync_msg').hide();
					batch_status = "none";
				}
			}
		});		
	}
	
	lMF.getStatistics = function(m = "cached"){
		laraAjax.doCall(LRAIAS_MODULE_ADMIN_LINK,{action: "getStatistics", mode : m});
	}

})(laraMainFunctions);

(function(lM){
	"use strict";
	
	function bootstrap_switch_action(id, state){
		let name = $(id).attr("name");
		if (name === "lr-panel-switch"){
			$(id).closest(".lr-groups-panel").find('select, input, textarea').prop("disabled", !state);
			$(id).closest(".lr-groups-panel").find('.lraias-bootstrap-switch').bootstrapSwitch("disabled", !state);
			$(id).bootstrapSwitch("disabled", false);

			if (state){
				$(id).closest(".lr-groups-panel").find('.lr-group-panel .lr-group-switch').each(function() {
					let group_state = $(this).bootstrapSwitch('state');
					$(this).closest(".lr-group-panel").find('select, input, textarea').prop("disabled", !group_state);
					$(this).closest(".lr-group-panel").find('.lraias-bootstrap-switch').bootstrapSwitch("disabled", !group_state);
					$(this).bootstrapSwitch("disabled", false);
				});
			}
		}
		
		if (name === "lr-group-switch" ){
			$(id).closest(".lr-group-panel").find('select, input, textarea').prop("disabled", !state);
			$(id).closest(".lr-group-panel").find('.lraias-bootstrap-switch').bootstrapSwitch("disabled", !state);
			$(id).bootstrapSwitch("disabled", false);
		}		

		if (name === "selectall-group-switch"){
			$('.lraias-bootstrap-switch').off('switchChange.bootstrapSwitch');
			$(id).closest('.box-body').find('.lr-group-selectall .lraias-bootstrap-switch').prop('checked', state).change();
			$('.lraias-bootstrap-switch').on('switchChange.bootstrapSwitch', switchChanged);
		}		
	}

	function switchChanged(event, state) {
		bootstrap_switch_action(this,state);
		setTimeout(function() {
			let settings_data = laraHelpers.formToObject("lraias_settings");
			laraAjax.doCall(LRAIAS_MODULE_ADMIN_LINK,{action: "save_settings", data: settings_data});
		}, 100);
	}

	function lrbootstrapSwitch() {
		$('.lraias-bootstrap-switch').bootstrapSwitch('destroy');
		$('.lraias-bootstrap-switch').bootstrapSwitch();
		$('.lraias-bootstrap-switch').on('switchChange.bootstrapSwitch', switchChanged);
	}	
	
	lM.construct = function (){
		if (typeof (LRAIAS_MODULE_DEBUG_MODE) !== "undefined"){laraHelpers.setDebugMode(LRAIAS_MODULE_DEBUG_MODE);}
		
		$( document ).ready(function() {
			
			lrbootstrapSwitch();
			$('a[data-toggle="pill"]').on('shown.bs.tab', function (e) {
				lrbootstrapSwitch();
			});			
			
			$('#lraias_settings').on('change', 'select, input, textarea', function () {
				let settings_data = laraHelpers.formToObject("lraias_settings");
				//let settings_data = $("#lraias_settings").serializeArray();
				laraAjax.doCall(LRAIAS_MODULE_ADMIN_LINK,{action: "save_settings", data: settings_data});				
			});			

			$('#lraias_refresh').on('click',function(){
				laraMainFunctions.getStatistics("live");
			});			
			
			$('#lraias_sync_kb').on('click',function(){
				laraMainFunctions.syncKB();
			});
			
			$("#lraias_errors").on('click', '#lraias_check_license', function () {
				let installation_id   = $(this).attr("lrdata-lcid");
				laraAjax.doCall(LRAIAS_MODULE_ADMIN_LINK,{action: "refreshlicense", id: installation_id}).done(function (data, textStatus, jqXHR) {
					if (data.status === "success"){
						laraMainFunctions.getStatistics();
					}
				});
			});	
			laraMainFunctions.getStatistics();	
		});
	}

})(laraMain);
